import pygame
from player import Player
from monkey import Monkey
from feather import Feather
from countdown import Countdown
from pygame.locals import *
from constants import *
from message import Message
from dropped import Dropped
import random
import data
import copy

class Game:
    '''Main game object'''
    def __init__(self):
        #initialize pygame
        pygame.init()
        pygame.display.set_mode(GAME_RES)
        self.screen = pygame.display.get_surface()
        self.clock = pygame.time.Clock()
        self.font = pygame.font.Font(data.font(),40) #the font

        #background image
        self.bg = data.load_image('sky.jpg')

        #keys being held down
        self.left = False
        self.right = False

        self.gamestage = MAIN_MENU
        self.menuimage = data.load_image('menu.jpg')

        self.reset() #start a new game

    def reset(self):
        self.gameover = False

        #sprites
        self.player = Player()
        self.monkeys = pygame.sprite.Group()
        self.feathers = pygame.sprite.Group()
        self.bgobjects = pygame.sprite.Group()
        self.fgobjects = pygame.sprite.Group()

        #timers
        self.feathertimer = Countdown(DEFAULT_FEATHER_TIME)
        self.monkeytimer = Countdown(DEFAULT_MONKEY_TIME)
        self.gameovertimer = Countdown(DEFAULT_TIME) #It's the FINAL COUNTDOWN duh nuh nuhhhhh nuhhhhhhh

        self._set_goal() #set a goal for the player

    def tick(self):
        '''Move stuff & make stuff happen'''
        self.clock.tick()
        dt = self.clock.get_time()
        ok = self._check_input()
        if not ok:
            return False
        if self.gamestage == PLAY_GAME and not self.gameover:
            if self.left:
                self.player.left()
            elif self.right:
                self.player.right()
            else:
                self.player.stop()
            self._move_stuff(dt)
            self._update_timers(dt)
        self._draw()
        return True

    def _move_stuff(self,dt):
        self.player.move(dt)
        for i in self.monkeys:
            i.move(dt)
            if pygame.sprite.collide_rect(i,self.player):
                i.change_direction()
                if not self.player.jumping:
                    for color,number in self.player.feathers.items():
                        for n in range(number):
                            self.fgobjects.add(Dropped(color,self.player.rect.midtop))
                    self.player.drop()
        for i in self.bgobjects:
            i.move(dt)
        for i in self.fgobjects:
            i.move(dt)
        for i in self.feathers:
            i.move(dt)
            if self.player.collide_top(i):
                feathers=copy.copy( self.player.feathers)
                if not self.player.add_feather(i):
                    self._fail() #wrong feather!!
                    for color,number in feathers.items():
                        for n in range(number):
                            self.fgobjects.add(Dropped(color,self.player.rect.midtop))

                i.kill() #remove the feather
                if self.player.aimless(): #goal is completed
                    self.fgobjects.add(Message('Hooray!',self.font,(0,150,0),0.5))
                    self._update_difficulty() #change the difficulty level
                    self.player.leftovertime += self.gameovertimer.time()
                    self._set_goal() #set a new goal for the player

    def _fail(self):
        '''display fail message and make feathers go flying everywhere'''
        self.fgobjects.add(Message('WRONG!',self.font,(255,0,0),0.5))

    def _set_goal(self):
        goal = {}
        for c in ['red','green','blue','yellow']:
            goal[c] = random.randint(0,5)
        self.player.set_goal(goal)

    def _update_timers(self,dt):
        '''Check to see if new stuff should be created'''
        if self.feathertimer.finished(dt):
            self.feathertimer.reset()
            self.feathers.add(Feather())
        if self.monkeytimer.finished(dt):
            self.monkeytimer.reset()
            self.monkeys.add(Monkey())
        if self.gameovertimer.finished(dt):
            self.gameover = True

    def _update_difficulty(self):
        '''Adjust the difficulty based on past success'''
        #time between monkeys is inversly proportional to the score
        #but there is also an additional term which is prortional to the number of drops
        quickness = self.player.leftovertime/float(self.player.score+1)
        time = max([DEFAULT_TIME-C_T1 * quickness,MIN_TIME])
        monkeytime = max([DEFAULT_MONKEY_TIME/(C_M1 * self.player.score+1) + C_M2 * self.player.drops,MIN_MONKEY_TIME])
        feathertime = max([DEFAULT_MONKEY_TIME/(C_F1 * self.player.score+1),MIN_FEATHER_TIME])
        self.gameovertimer = Countdown(time)
        self.monkeytimer = Countdown(monkeytime)
        self.feathertime = Countdown(feathertime)

    def _draw(self):
        '''Draw everything'''
        if self.gamestage == MAIN_MENU:
            self.screen.blit(self.menuimage,self.menuimage.get_rect())
            pygame.display.flip()
        else:
            self.screen.blit(self.bg,self.bg.get_rect())
            for i in self.bgobjects:
                i.draw(self.screen)
            for i in self.monkeys:
                i.draw(self.screen)
            for i in self.feathers:
                i.draw(self.screen)
            self.player.draw(self.screen)
            for i in self.fgobjects:
                i.draw(self.screen)
            self.draw_scorestuff()
            pygame.display.flip()

    def draw_scorestuff(self):
        #draw level in the top left
        level = str(self.player.score/2+1)
        surface = self.font.render("Level "+level,True,(0,0,0))
        rect = surface.get_rect()
        rect.topleft = (20,20)
        self.screen.blit(surface,rect)

        #draw goal on the right
        y = 100
        for color,number in self.player.goal.items():
            if number > 0:
                if color in self.player.feathers:
                    fetched = self.player.feathers[color]
                else:
                    fetched = 0
                surface = self.font.render(str(fetched)+'/'+str(number),True,(0,0,0))
                rect = surface.get_rect()
                rect.topright = (GAME_RES[0]-20,y)
                self.screen.blit(surface,rect)

                surface = pygame.Surface((10,10))
                surface.fill(pygame.Color(color))
                rect2 = surface.get_rect()
                rect2.centery = rect.centery
                rect2.right = rect.left - 20
                self.screen.blit(surface,rect2)
                y = rect.bottom + 50

        if self.gameover:
            surface = self.font.render("Game over!",True,(0,0,0))
            rect = surface.get_rect()
            rect.center = (GAME_RES[0]/2,GAME_RES[1]/2)
            self.screen.blit(surface,rect)
        else:
            surface = self.font.render(str(int(self.gameovertimer.time())),True,(0,0,0))
            rect = surface.get_rect()
            rect.topleft = (20,60)
            self.screen.blit(surface,rect)

    def _check_input(self):
        '''Check to see if any buttons have been pressed'''
        for event in pygame.event.get():
            if event.type == QUIT:
                return False
            elif event.type == KEYDOWN:
                if self.gamestage == MAIN_MENU or self.gameover:
                    self.gamestage = PLAY_GAME
                    self.gameover = False
                elif event.key == K_LEFT:
                    self.left = True
                elif event.key == K_RIGHT:
                    self.right = True
                elif event.key == K_UP:
                    if not self.player.jumping and not self.player.falling:
                        self.player.jump()
            elif event.type == KEYUP:
                if event.key == K_LEFT:
                    self.left = False
                elif event.key == K_RIGHT:
                    self.right = False
        return True

    def fps(self):
        '''Return the frame rate'''
        return self.clock.get_fps

